@@ -1,4 +1,3 @@
-import math
 import multiprocessing
 from functools import partial
 import random
@@ -11,7 +10,8 @@ from typing import Any, TypeVar, Iterable, Iterator, List, Callable, Optional, U
 
 T = TypeVar('T')
 
-__all__ = ['ThreadedIterator', 'MultiWorkerCallableIterator', 'BufferedIterator', 'DoubleBufferedIterator', 'shuffled_iterator', 'uniform_sample_iterator']
+__all__ = ['ThreadedIterator', 'MultiWorkerCallableIterator', 'BufferedIterator', 'DoubleBufferedIterator', 'shuffled_iterator',
+           'subsample_iterator', 'uniform_sample_iterator']
 
 
 class ThreadedIterator(Iterator[T]):
@@ -258,4 +258,37 @@ def uniform_sample_iterator(input_iterator: Iterator[T], sample_size: int, rng:
         if j < sample_size:
             buffer[j] = element
 
-    return buffer
\ No newline at end of file
+    return buffer
+
+
+def subsample_iterator(
+        input_iter: Iterator[T],
+        sampling_rate: float,
+        num_elements: Optional[int]=None,
+        rng: Optional[random.Random]=None
+) -> Iterator[T]:
+    """
+    Stream-sample an input iterator with some sampling rate. This wrapper simply iterates `input_iter` and subsamples its
+        elements until `num_elements` have been returned or the iterator has been exhausted.
+
+    :param input_iter: the original iterator
+    :param sampling_rate: the sampling rate in (0,1].
+    :param num_elements: the number of elements to yield.
+    :param rng:
+    :return:
+    """
+    if rng is None:
+        rng = random
+
+    if sampling_rate == 1.0 and num_elements is not None:
+        yield from islice(input_iter, num_elements)
+    elif sampling_rate == 1.0:
+        yield from input_iter
+    else:
+        num_taken = 0
+        for element in input_iter:
+            if rng.random() < sampling_rate:
+                yield element
+                num_taken += 1
+                if num_elements is not None and num_taken >= num_elements:
+                    break
